from zs_utils.api.ozon.base_api import OzonAPI


class OzonGetProductListAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_GetProductList
    """

    resource_method = "v2/product/list"
    allowed_params = [
        "filter",
        "last_id",
        "limit",
    ]


class OzonGetProductsInfoAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_GetProductInfoListV2
    """

    resource_method = "v2/product/info/list"
    allowed_params = ["offer_id", "product_id", "sku"]


class OzonGetProductsAttributeAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_GetProductAttributesV3
    """

    resource_method = "v3/products/info/attributes"
    required_params = ["limit"]
    allowed_params = [
        "filter",
        "last_id",
        "sort_by",
        "sort_dir",
    ]


class OzonGetProductsFBSStocksAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_ProductStocksByWarehouseFbs
    """

    resource_method = "v1/product/info/stocks-by-warehouse/fbs"
    required_params = ["fbs_sku"]


class OzonGetProductStatusAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_GetImportProductsInfo
    """

    resource_method = "v1/product/import/info"
    required_params = ["task_id"]


class OzonCreateProductAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_ImportProductsV3
    """

    resource_method = "v3/product/import"
    required_params = ["items"]


class OzonUpdateProductStocksAPI(OzonAPI):
    resource_method = "v1/product/import/stocks"
    required_params = ["stocks"]


class OzonUpdateProductWarehouseStocksAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_ProductsStocksV2
    """

    resource_method = "v2/products/stocks"
    required_params = ["stocks"]


class OzonArchiveProductAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_ProductArchive
    """

    resource_method = "v1/product/archive"
    required_params = ["product_id"]


class OzonDeleteProductFromArchiveAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_DeleteProducts
    """

    resource_method = "v2/products/delete"
    required_params = ["products"]


class OzonReturnProductFromArchiveAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_ProductUnarchive
    """

    resource_method = "v1/product/unarchive"
    required_params = ["product_id"]


class OzonUpdateProductPricesAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ProductAPI_ImportProductsPrices
    """

    resource_method = "v1/product/import/prices"
    required_params = ["prices"]



from zs_utils.api.aliexpress.base_api import AliexpressAPI


class OrderFulfillAPI(AliexpressAPI):
    """
    Docs: https://developers.aliexpress.com/en/doc.htm?docId=42269&docType=2
    """

    resource_method = "aliexpress.solution.order.fulfill"
    required_params = ["service_name", "out_ref", "send_type", "logistics_no"]
    allowed_params = ["tracking_website", "description"]


class GetOrderListAPI(AliexpressAPI):
    """
    Docs: https://developers.aliexpress.com/en/doc.htm?docId=42270&docType=2
    """

    resource_method = "aliexpress.solution.order.get"
    allowed_params = ["param0"]


class GetOrderInfoAPI(AliexpressAPI):
    """
    Docs: https://developers.aliexpress.com/en/doc.htm?docId=42707&docType=2
    """

    resource_method = "aliexpress.solution.order.info.get"
    allowed_params = ["param1"]


class GetOrderReceiptInfoAPI(AliexpressAPI):
    """
    Docs: https://developers.aliexpress.com/en/doc.htm?docId=42369&docType=2
    """

    resource_method = "aliexpress.solution.order.receiptinfo.get"
    allowed_params = ["param1"]



MarketplaceToCurrency = {
    "default": "USD",
    "EBAY_US": "USD",
    "EBAY_MOTORS_US": "USD",
    "EBAY_CA": "CAD",
    "EBAY_GB": "GBP",
    "EBAY_AU": "AUD",
    "EBAY_AT": "EUR",
    "EBAY_BE_FR": "EUR",
    "EBAY_FR": "EUR",
    "EBAY_DE": "EUR",
    "EBAY_IT": "EUR",
    "EBAY_BE_NL": "EUR",
    "EBAY_NL": "EUR",
    "EBAY_ES": "EUR",
    "EBAY_CH": "EUR",
    "EBAY_TW": "TWD",
    "EBAY_CZ": "EUR",
    "EBAY_DK": "EUR",
    "EBAY_FI": "EUR",
    "EBAY_GR": "EUR",
    "EBAY_HK": "HKD",
    "EBAY_HU": "EUR",
    "EBAY_IN": "INR",
    "EBAY_ID": "IDR",
    "EBAY_IE": "EUR",
    "EBAY_IL": "ILS",
    "EBAY_MY": "MYR",
    "EBAY_NZ": "NZD",
    "EBAY_NO": "EUR",
    "EBAY_PH": "PHP",
    "EBAY_PL": "EUR",
    "EBAY_PT": "EUR",
    "EBAY_PR": "USD",
    "EBAY_RU": "RUB",
    "EBAY_SG": "SGD",
    "EBAY_ZA": "ZAR",
    "EBAY_SE": "EUR",
    "EBAY_TH": "THB",
    "EBAY_VN": "VND",
    "EBAY_CN": "CNY",
    "EBAY_PE": "PEN",
    "EBAY_CA_FR": "CAD",
    "EBAY_JP": "JPY",
}



from datetime import datetime
from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetEbayAccountBillingInfo",
    "GetEbaySellingInfo",
    "GetEbaySellingSummary",
]


class GetEbayAccountBillingInfo(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/GetAccount.html
    """

    method_name = "GetAccount"

    def get_params(
        self,
        account_history_selection: str,
        begin_date: datetime = None,  # 4 months ago -- ebay limit
        end_date: datetime = None,
        **kwargs,
    ):
        assert account_history_selection in [
            "BetweenSpecifiedDates",
            "LastInvoice",
        ], "Недопустимое значение 'account_history_selection'."

        if account_history_selection == "BetweenSpecifiedDates":
            assert begin_date and end_date, "Необходимо задать временной диапазон ('begin_date' и 'end_date')."

        return {
            "AccountHistorySelection": account_history_selection,
            # "AccountEntrySortType": "AccountEntryCreatedTimeDescending",
            "BeginDate": begin_date,
            "EndDate": end_date,
            "ExcludeBalance": False,
            "ExcludeSummary": False,
        }


class GetEbaySellingInfo(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetMyeBaySelling.html
    """

    method_name = "GetMyeBaySelling"

    containers = [
        "ActiveList",
        "DeletedFromSoldList",
        "DeletedFromUnsoldList",
        "ScheduledList",
        "SoldList",
        "UnsoldList",
        "SellingSummary",
    ]

    def get_params(self, **kwargs):
        return {container: {"Include": kwargs.get(container, False)} for container in self.containers}


class GetEbaySellingSummary(GetEbaySellingInfo):
    def get_params(self, **kwargs):
        kwargs.update({container: False for container in self.containers})
        kwargs["SellingSummary"] = True
        return super().get_params(**kwargs)



__all__ = [
    "convert_from_nested_dict_to_plain_dict",
    "convert_from_plain_dict_to_nested_dict",
]


def convert_from_nested_dict_to_plain_dict(data=None, parent_string=None):
    result = {}

    if isinstance(data, dict):
        for key, value in data.items():
            if isinstance(value, dict) or isinstance(value, list):
                result.update(
                    convert_from_nested_dict_to_plain_dict(
                        value,
                        parent_string + "." + str(key) if parent_string else str(key),
                    )
                )
            else:
                pre_parent_string = parent_string + "." if parent_string else ""
                result[pre_parent_string + str(key)] = value

    elif isinstance(data, list):
        for key, value in enumerate(data):
            if isinstance(value, dict) or isinstance(value, list):
                result.update(
                    convert_from_nested_dict_to_plain_dict(
                        value,
                        parent_string + "[" + str(key) + "]" if parent_string else "[" + str(key) + "]",
                    )
                )
            else:
                pre_parent_string = parent_string if parent_string else ""
                result[pre_parent_string + "[" + str(key) + "]"] = value

    return result


def convert_from_plain_dict_to_nested_dict(data=None):
    if not data:
        return data

    if list(data.keys())[0].startswith("["):
        result = list()
    else:
        result = dict()

    last_key = None
    ignored_keys = []
    for key, value in data.items():
        if key in ignored_keys:
            continue
        elif key == "":
            return value
        elif key.startswith("["):
            rsb_idx = key.index("]")
            if last_key != key[1:rsb_idx]:
                last_key = key[1:rsb_idx]
                new_data = {k[rsb_idx + 1 :]: v for k, v in data.items() if k[1 : k.index("]")] == key[1:rsb_idx]}
                result.insert(
                    int(key[1:rsb_idx]),
                    convert_from_plain_dict_to_nested_dict(new_data),
                )
        elif key.startswith("."):
            if last_key != key[0]:
                last_key = key[0]
                new_data = {k[1:]: v for k, v in data.items() if k[0] == key[0]}
                result.update(convert_from_plain_dict_to_nested_dict(new_data))
        else:
            for index, char in enumerate(key):
                if char == "[" or char == ".":
                    if last_key != key[0:index]:
                        last_key = key[0:index]

                        new_data = {}
                        for k, v in data.items():
                            if k[0:index] == last_key:
                                new_data[k[index:]] = v
                                ignored_keys.append(k)

                        result[last_key] = convert_from_plain_dict_to_nested_dict(new_data)

                    break
            else:
                result[key] = value

    return result



from .abstract import AbstractModel, XMLElement


class Option(AbstractModel):
    """
    Option model for both the delivery option and the pickup option.

    Docs:
    https://yandex.ru/support/partnermarket/elements/delivery-options.html
    https://yandex.ru/support/partnermarket/elements/pickup-options.html
    """

    __slots__ = ["cost", "days", "order_before"]

    def __init__(self, cost, days, order_before=None):
        self.cost = cost
        self.days = days
        self.order_before = order_before

    def create_dict(self, **kwargs) -> dict:
        return dict(cost=self.cost, days=self.days, order_before=self.order_before)

    def create_xml(self, **kwargs) -> XMLElement:
        attribs = {"cost": self.cost, "days": self.days}
        if self.order_before:
            attribs["order-before"] = self.order_before
        el = XMLElement("option", attrib=attribs)
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Option":
        el.attrib["order_before"] = el.attrib.pop("order-before", None)
        return Option(**el.attrib)



from zs_utils.api.ebay.base_api import EbayAPI


class BulkMigrateListing(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/listing/methods/bulkMigrateListing
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/bulk_migrate_listing"



from zs_utils.api.aliexpress_russia.base_api import AliexpressRussiaAPI


class GetAliexpressCategoryTreeAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/categories#heading-poluchit-kategorii-verhnego-urovnya
    """

    resource_method = "/api/v1/categories/top"


class GetAliexpressCategoryInfoAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/categories#heading-poluchit-atributi-kategorii
    """

    resource_method = "/api/v1/categories/get"
    required_params = ["ids"]


class GetAliexpressAttributeValuesAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/categories#heading-poluchit-informatsiyu-o-znacheniyah-atributa
    """

    resource_method = "/api/v1/categories/values-dictionary"
    required_params = ["category_id", "property_id", "is_sku_property"]


class GetAliexpressBrandsAPI(AliexpressRussiaAPI):
    """
    https://business.aliexpress.ru/docs/brand
    """

    resource_method = "/api/v1/brand/get-brand-list"
    required_params = ["offset", "limit"]
    allowed_params = ["filters", "sort"]



from model_utils import Choices


AvailabilityTypeEnum = Choices(
    ("IN_STOCK", "В наличии"),
    ("OUT_OF_STOCK", "Нет в наличии"),
    ("SHIP_TO_STORE", "Ожидается пополнение"),
)

PackageTypeEnum = Choices(
    ("LETTER", "Бумага"),
    ("BULKY_GOODS", "Bulky good"),
    ("CARAVAN", "Caravan"),
    ("CARS", "Автомобиль"),
    ("EUROPALLET", "Euro pallet"),
    ("EXPANDABLE_TOUGH_BAGS", "Expandable tough bag"),
    ("EXTRA_LARGE_PACK", "Extra large pack"),
    ("FURNITURE", "Мебель"),
    ("INDUSTRY_VEHICLES", "Industry vehicle"),
    ("LARGE_CANADA_POSTBOX", "A Canada Post large box"),
    ("LARGE_CANADA_POST_BUBBLE_MAILER", "Canada Post large bubble mailer"),
    ("LARGE_ENVELOPE", "Большой конверт"),
    ("MAILING_BOX", "Mailing box"),
    ("MEDIUM_CANADA_POST_BOX", "Medium Canada Post box"),
    ("MEDIUM_CANADA_POST_BUBBLE_MAILER", "Medium Canada Post bubble mailer"),
    ("MOTORBIKES", "Мотоцикл"),
    ("ONE_WAY_PALLET", "One-way pallet"),
    ("PACKAGE_THICK_ENVELOPE", "Толстый конверт"),
    ("PADDED_BAGS", "Padded bag"),
    ("PARCEL_OR_PADDED_ENVELOPE", "Посылка или мягкий конверт"),
    ("ROLL", "Roll"),
    ("SMALL_CANADA_POST_BOX", "Small Canada Post box"),
    ("SMALL_CANADA_POST_BUBBLE_MAILER", "Small Canada Post bubble mailer"),
    ("TOUGH_BAGS", "Tough bag"),
    ("UPS_LETTER", "Письмо UPS"),
    ("USPS_FLAT_RATE_ENVELOPE", "USPS flat-rate envelope"),
    ("USPS_LARGE_PACK", "USPS large pack"),
    ("VERY_LARGE_PACK", "USPS very large pack"),
    ("WINE_PAK", "Wine pak"),
)

ShippingServiceTypeEnum = Choices(
    ("DOMESTIC", "Внутренняя доставка"),
    ("INTERNATIONAL", "Международная доставка"),
)

SoldOnEnum = Choices(
    ("ON_EBAY", "Товар продавался по указанной цене на сайте eBay"),
    ("OFF_EBAY", "Товар продавался по указанной цене на сторонних сайтах"),
    (
        "ON_AND_OFF_EBAY",
        "Товар продавался по указанной цене как на сайте eBay, так и на сторонних сайтах",
    ),
)

MinimumAdvertisedPriceHandlingEnum = Choices(
    ("NONE", "Не использовать"),
    ("PRE_CHECKOUT", "До оформления заказа"),
    ("DURING_CHECKOUT", "После оформления заказа"),
)

# https://developer.ebay.com/api-docs/sell/inventory/types/slr:ListingStatusEnum
ListingStatusEnum = Choices(
    ("ACTIVE", "ACTIVE"),
    ("OUT_OF_STOCK", "OUT_OF_STOCK"),
    ("INACTIVE", "INACTIVE"),
    ("ENDED", "ENDED"),
    ("EBAY_ENDED", "EBAY_ENDED"),
    ("NOT_LISTED", "NOT_LISTED"),
)



from typing import Optional

from zs_utils.yandex_market_language.exceptions import ValidationError


__all__ = [
    "EnableAutoDiscountField",
]


ENABLE_AUTO_DISCOUNTS_CHOICES = ("yes", "true", "1", "no", "false", "0")


class EnableAutoDiscountField:
    _enable_auto_discounts = None

    @property
    def enable_auto_discounts(self) -> Optional[bool]:
        if self._enable_auto_discounts:
            if self._enable_auto_discounts in ["yes", "true", "1"]:
                return True
            elif self._enable_auto_discounts in ["no", "false", "0"]:
                return False
        return None

    @enable_auto_discounts.setter
    def enable_auto_discounts(self, value):
        if value in ["yes", "true", "1", "no", "false", "0"]:
            self._enable_auto_discounts = value
        elif value is True:
            self._enable_auto_discounts = "true"
        elif value is False:
            self._enable_auto_discounts = "false"
        elif value is None:
            self._enable_auto_discounts = value
        else:
            raise ValidationError(
                (
                    "enable_auto_discounts should be True, False "
                    "or str from available values: {values}".format(values=", ".join(ENABLE_AUTO_DISCOUNTS_CHOICES))
                )
            )



from yookassa import Configuration
from yookassa.domain.common import ResponseObject

from zs_utils.api.base_action import APIAction


class YooKassaAction(APIAction):
    description = "Взаимодействие с API Yoo Kassa"
    save_notifications = True
    api_method = None
    VALIDATE_PARAMS = True

    def __init__(self, instance=None, propagate_exception: bool = False, **kwargs):
        super().__init__(instance, propagate_exception, **kwargs)

        self.configured = False

    def copy_instance(self, instance, **kwargs) -> None:
        super().copy_instance(instance, **kwargs)

        self.user = instance.user
        self.test_mode = instance.test_mode
        self.credentials = instance.credentials

    def set_action_variables(self, **kwargs) -> None:
        super().set_action_variables(**kwargs)
        self.user = kwargs.get("user")
        self.credentials = kwargs.get("credentials")
        self.test_mode = kwargs.get("test_mode")

    @classmethod
    def clean_data(cls, data: dict):
        cleaned_data = {}
        for key, value in data.items():
            if value or type(value) in [bool, int]:
                if isinstance(value, dict):
                    cleaned_data.update({key: cls.clean_data(value)})
                else:
                    cleaned_data.update({key: value})
        return cleaned_data

    def get_params(self, **kwargs) -> dict:
        if "params" in self.required_params:
            if kwargs.get("params"):
                kwargs["params"] = self.clean_data(data=kwargs["params"])
            else:
                kwargs["params"] = {}
        return super().get_params(**kwargs)

    def configure_yookassa(self):
        if self.test_mode:
            account_id = self.credentials["test"]["account_id"]
            secret_key = self.credentials["test"]["secret_key"]
        else:
            account_id = self.credentials["production"]["account_id"]
            secret_key = self.credentials["production"]["secret_key"]

        Configuration.configure(account_id, secret_key)

        self.configured = True

    def before_request(self, **kwargs):
        super().before_request(**kwargs)

        self.configure_yookassa()

    def make_request(self, **kwargs):
        if not getattr(self, "api_method", None):
            raise NotImplementedError("Для обращения к API YooKassa необходимо задать атрибут 'api_method'.")

        assert self.configured, "Метод Configuration.configure не был вызван."

        try:
            response: ResponseObject = self.api_method(**kwargs)
        except Exception as error:
            if error.args and isinstance(error.args[0], dict):
                errors: dict = error.args[0]
                message = f'{errors["description"]}. Code: {errors["code"]}.'
                raise self.exception_class(message=message, message_dict=errors)
            else:
                raise self.exception_class(message=str(error))

        return {"results": dict(response)}



from zs_utils.api.base_api import BaseAPI
from zs_utils.api.ebay.utils import custom_quote


__all__ = [
    "EbayAPI",
]


class EbayAPI(BaseAPI):
    production_api_url = "https://api.ebay.com/"
    sandbox_api_url = "https://api.sandbox.ebay.com/"

    params_to_quote = [
        "sku",
        "inventoryItemGroupKey",
        "merchantLocationKey",
    ]

    # TODO: валидация?
    MAX_LIMIT = 100
    MAX_OFFSET = 99

    def __init__(self, access_token: str, marketplace_id: str = None, **kwargs):
        super().__init__(**kwargs)

        self.access_token = access_token
        self.marketplace_id = marketplace_id

    @property
    def headers(self):
        # DOCS: https://developer.ebay.com/api-docs/static/rest-request-components.html#marketpl
        DOMAIN_TO_LOCALE = {
            "EBAY_AT": "de-AT",
            "EBAY_AU": "en-AU",
            "EBAY_BE": "fr-BE",
            "EBAY_CA": "en-CA",
            "EBAY_CH": "de-CH",
            "EBAY_DE": "de-DE",
            "EBAY_ES": "es-ES",
            "EBAY_FR": "fr-FR",
            "EBAY_GB": "en-GB",
            "EBAY_HK": "zh-HK",
            "EBAY_IE": "en-IE",
            "EBAY_IN": "en-GB",
            "EBAY_IT": "it-IT",
            "EBAY_MY": "en-MY",
            "EBAY_NL": "nl-NL",
            "EBAY_PH": "en-PH",
            "EBAY_PL": "pl-PL",
            "EBAY_TH": "th-TH",
            "EBAY_TW": "zh-TW",
        }
        locale = DOMAIN_TO_LOCALE.get(self.marketplace_id)
        if not locale:
            locale = "en-US"
        return {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Language": locale,
            "Content-Type": "application/json",
            "Accept-Language": locale,
            "Accept": "application/json",
            "Accept-Charset": "utf-8",
            "Accept-Encoding": "application/gzip",
            "X-EBAY-C-MARKETPLACE-ID": self.marketplace_id,
        }

    def build_url(self, params: dict):
        next_url = params.pop("next_url", None)
        if next_url:
            url = self.sandbox_api_url if self.is_sandbox else self.production_api_url
            if next_url.startswith("/"):
                url += next_url
            else:
                url = next_url
            return url
        else:
            return super().build_url(params=params)

    def get_path_params(self, params: dict):
        path_params = super().get_path_params(params)
        return {
            param: custom_quote(value)
            for param, value in path_params.items()
            if value and (param in self.params_to_quote)
        }

    def get_clean_params(self, params: dict) -> dict:
        clean_params = {
            "next_url": params.get("next_url"),
            "payload": params.get("payload"),
        }
        if not params.get("next_url"):
            clean_params.update(super().get_clean_params(params))

            for param in self.params_to_quote:
                if clean_params.get(param):
                    clean_params[param] = custom_quote(clean_params[param])

        return clean_params

    def get_payload(self, params: dict):
        return params.pop("payload", None)



from typing import Optional

from .abstract import AbstractModel, XMLElement

from zs_utils.yandex_market_language.exceptions import ValidationError


CURRENCY_CHOICES = ("RUR", "RUB", "UAH", "BYN", "KZT", "USD", "EUR")
RATE_CHOICES = ("CBRF", "NBU", "NBK", "CB")


class Currency(AbstractModel):
    """
    Currency model.
    Used to create a list of shop currency rates.

    Docs:
    https://yandex.ru/support/partnermarket/elements/currencies.html
    """

    __slots__ = ["_currency", "_rate", "_plus"]

    def __init__(self, currency="RUR", rate: str = 1, plus=None):
        self.currency = currency
        self.rate = rate
        self.plus = plus

    @property
    def currency(self) -> str:
        return self._currency

    @currency.setter
    def currency(self, value):
        if value not in CURRENCY_CHOICES:
            raise ValidationError(
                "Price data is accepted only in: {formatted_choices}".format(
                    formatted_choices=", ".join(CURRENCY_CHOICES)
                )
            )
        self._currency = value

    @property
    def rate(self) -> str:
        return self._rate

    @rate.setter
    def rate(self, value):
        if value not in RATE_CHOICES:
            try:
                float(value)
            except (TypeError, ValueError):
                raise ValidationError(
                    (
                        "The rate parameter can have the following values: "
                        "number (int or float), {rate_choices}".format(rate_choices=", ".join(RATE_CHOICES))
                    )
                )

        self._rate = str(value)

    @property
    def plus(self) -> Optional[str]:
        return self._plus

    @plus.setter
    def plus(self, value):
        self._plus = self._is_valid_int(value, "plus", True)

    def create_dict(self, **kwargs) -> dict:
        return dict(id=self.currency, rate=self.rate, plus=self.plus)

    def create_xml(self, **kwargs) -> XMLElement:
        el = XMLElement("currency", self.clean_dict)
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Currency":
        el.attrib["currency"] = el.attrib.pop("id")
        return Currency(**el.attrib)



from zs_utils.api.etsy.base_api import EtsyAPI


class GetEtsyCategories(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getSellerTaxonomyNodes
    """

    description = "Получение категорий товаров Etsy"
    http_method = "GET"
    resource_method = "seller-taxonomy/nodes"


class GetEtsyCategoryAspects(EtsyAPI):
    """
    Docs:
    https://www.etsy.com/openapi/developers#operation/getPropertiesByTaxonomyId
    """

    description = "Получение проперти для категории товаров Etsy"
    http_method = "GET"
    resource_method = "seller-taxonomy/nodes/{taxonomy_id}/properties"
    required_params = ["taxonomy_id"]



from django.contrib.auth import get_user_model
from rest_framework import serializers


__all__ = [
    "UserLightSerializer",
]

User = get_user_model()


class UserLightSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = [
            "id",
            "identifier",
            "email",
        ]



import requests
from urllib.parse import urlparse, parse_qsl
from pprint import pprint
from django.conf import settings
from django.utils import timezone

from zs_utils.api.kokoc_crm import constants


__all__ = [
    "KokocCRMService",
]


class KokocCRMService:
    @classmethod
    def _make_request(cls, method: str, path: str, params: dict = None):
        assert method in ("get", "post", "put", "delete"), f"Недопустимый метод '{method}'"

        request_data = {
            "method": method,
            "url": f"https://my.kokocgroup.ru/rest/2806/{settings.KOKOC_CRM_TOKEN}/{path}",
        }
        if params:
            if params.get("fields"):
                params["fields"] = {k: v for k, v in params["fields"].items() if v}
            params = cls._prepare_params(params=params).encode("utf-8")
            if method == "get":
                request_data["params"] = params
            else:
                request_data["data"] = params

        response = requests.request(**request_data)
        pprint({k: v for k, v in response.request.__dict__.items() if k in ["method", "url", "body"]})
        results = response.json()

        if not response.ok:
            raise requests.exceptions.HTTPError(results["error_description"])

        return {"data": results, "response": response}

    @classmethod
    def _prepare_params(cls, params, prev="") -> str:
        """
        Конвертация данных в массив битрикс
        Source: https://github.com/akopdev/bitrix24-python-rest/blob/master/bitrix24/bitrix24.py
        """
        ret = ""
        if isinstance(params, dict):
            for key, value in params.items():
                if isinstance(value, dict):
                    if prev:
                        key = "{0}[{1}]".format(prev, key)
                    ret += cls._prepare_params(value, key)
                elif (isinstance(value, list) or isinstance(value, tuple)) and len(value) > 0:
                    for offset, val in enumerate(value):
                        if isinstance(val, dict):
                            ret += cls._prepare_params(val, "{0}[{1}][{2}]".format(prev, key, offset))
                        else:
                            if prev:
                                ret += "{0}[{1}][{2}]={3}&".format(prev, key, offset, val)
                            else:
                                ret += "{0}[{1}]={2}&".format(key, offset, val)
                else:
                    if prev:
                        ret += "{0}[{1}]={2}&".format(prev, key, value)
                    else:
                        ret += "{0}={1}&".format(key, value)
        return ret

    @classmethod
    def _list_objects(cls, obj: constants.CRM_OBJECTS, **filter_params) -> dict:
        if obj == constants.CRM_OBJECTS.CONTACT:
            path = "crm.contact.list.json"
        elif obj == constants.CRM_OBJECTS.DEAL:
            path = "crm.deal.list.json"
        elif obj == constants.CRM_OBJECTS.STATUS:
            path = "crm.status.list.json"
        else:
            raise NotImplementedError(f'Получение списка объектов "{constants.CRM_OBJECTS[obj]}" не поддерживается.')

        results = cls._make_request(method="get", path=path, params={"filter": filter_params})
        results["data"] = results["data"]["result"]
        return results

    @classmethod
    def _get_object(cls, obj: constants.CRM_OBJECTS, remote_id: int) -> dict:
        if obj == constants.CRM_OBJECTS.CONTACT:
            path = "crm.contact.get.json"
        elif obj == constants.CRM_OBJECTS.DEAL:
            path = "crm.deal.get.json"
        elif obj == constants.CRM_OBJECTS.STATUS:
            path = "crm.status.get.json"
        else:
            raise NotImplementedError(f'Получение объекта "{constants.CRM_OBJECTS[obj]}" не поддерживается.')

        results = cls._make_request(method="get", path=path, params={"id": remote_id})
        results["data"] = results["data"]["result"]
        return results

    @classmethod
    def get_deal(cls, deal_id: int) -> dict:
        return cls._get_object(obj=constants.CRM_OBJECTS.DEAL, remote_id=deal_id)

    @classmethod
    def get_contact(cls, contact_id: int) -> dict:
        return cls._get_object(obj=constants.CRM_OBJECTS.CONTACT, remote_id=contact_id)

    @classmethod
    def create_contact(
        cls,
        name: str,
        phone: str = None,
        email: str = None,
    ) -> dict:
        """
        Создание контакта в системе кокос
        """
        params = {
            "fields": {
                "NAME": name,
                "PHONE": [{"VALUE": phone}] if phone else None,
                "EMAIL": [{"VALUE": email}] if email else None,
            },
        }
        return cls._make_request(method="post", path="crm.contact.add.json", params=params)

    @classmethod
    def _get_utm_params_from_url(cls, url: str) -> dict:
        parsed_url = urlparse(url=url)
        return {k: v for k, v in parse_qsl(qs=parsed_url.query) if k.startswith("utm_")}

    @classmethod
    def create_deal(
        cls,
        title: str,
        stage: str,
        contact_id: int = None,
        comment: str = None,
        source: str = None,
        assigned_by: str = "2806",
        department: str = "9923",
        utm_params: dict = None,
    ) -> dict:
        """
        Создание сделки в системе кокос
        """
        if utm_params:
            utm_params = {
                key: utm_params.get(key.lower())
                for key in [
                    "UTM_SOURCE",
                    "UTM_MEDIUM",
                    "UTM_CAMPAIGN",
                    "UTM_CONTENT",
                    "UTM_TERM",
                ]
            }
        else:
            utm_params = {}
        params = {
            "fields": {
                "TITLE": title,
                "CATEGORY_ID": "22",
                "STAGE_ID": stage,
                "CONTACT_ID": contact_id,
                "COMMENTS": comment,
                "UF_CRM_DEAL_AMO_OKDUOUUHNSIOPEJZ": source,
                "ASSIGNED_BY_ID": assigned_by,
                "UF_DEPARTMENT": department,
                **utm_params,
            },
        }
        return cls._make_request(method="post", path="crm.deal.add.json", params=params)

    @classmethod
    def update_deal_stage(cls, deal_id: int, stage: str) -> dict:
        params = {
            "id": deal_id,
            "fields": {
                "STAGE_ID": stage,
            },
        }
        return cls._make_request(method="post", path="crm.deal.update.json", params=params)

    @classmethod
    def delete_deal(cls, deal_id: int) -> dict:
        return cls._make_request(method="post", path="crm.deal.delete.json", params={"id": deal_id})

    @classmethod
    def create_deal_todo(cls, deal_id: int, text: str, deadline: timezone.datetime, responsible_id: int = None):
        return KokocCRMService._make_request(
            method="post",
            path="crm.activity.todo.add",
            params={
                "ownerTypeId": 2,  # значение 2 означает сделку
                "ownerId": deal_id,
                "description": text,
                # Дедлайн по московскому времени
                "deadline": deadline.astimezone(timezone.timezone(offset=timezone.timedelta(hours=3))).strftime(
                    "%d.%m.%Y %H:%M:%S"
                ),
                "responsibleId": responsible_id,
            },
        )



from typing import Optional

from .abstract import AbstractModel, XMLElement


class AbstractPrice(AbstractModel):
    tag_name = None

    __slots__ = ["_value", "_is_starting"]

    def __init__(self, value=None, is_starting=False):
        if not self.tag_name:
            raise ValueError("Необходимо определить атрибут 'tag_name'.")

        self.value = value
        self.is_starting = is_starting

    @property
    def is_starting(self) -> Optional[bool]:
        return self._str_to_bool(self._is_starting)

    @is_starting.setter
    def is_starting(self, value):
        self._is_starting = self._is_valid_bool(value, "is_starting", True)

    @property
    def value(self) -> float:
        return float(self._value)

    @value.setter
    def value(self, v):
        if isinstance(v, str):
            v = v.replace(",", ".")
        self._value = self._is_valid_float(v, self.tag_name)

    def create_dict(self, **kwargs) -> dict:
        return dict(value=self.value, is_starting=self.is_starting)

    def create_xml(self, **kwargs) -> XMLElement:
        el = XMLElement(self.tag_name)

        if self.is_starting:
            el.attrib["from"] = "true"

        el.text = self._value
        return el

    @classmethod
    def from_xml(cls, el: XMLElement) -> "AbstractModel":
        return cls(el.text, el.attrib.get("from", False))


class Price(AbstractPrice):
    """
    Actual offer price model.
    """

    tag_name = "price"


class OldPrice(AbstractPrice):
    """
    Actual offer price model.
    """

    tag_name = "oldprice"



from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetEbayDetails",
]


class GetEbayDetails(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GeteBayDetails.html
    Available detail names:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/types/DetailNameCodeType.html
    """

    method_name = "GeteBayDetails"

    def get_params(self, detail_names: list, **kwargs):
        if detail_names:
            if not isinstance(detail_names, list):
                detail_names = [detail_names]
        else:
            raise AttributeError('Необходимо задать параметр "detail_names"')

        return {"DetailName": detail_names}
